-- Copyright 2010-2016 by Adin Hunter Baber, all rights reserved
local QuietMacrosName, QuietMacros = ...
-- lua api
-- blizzard api
local DEFAULT_CHAT_FRAME = DEFAULT_CHAT_FRAME
local CheckInteractDistance = CheckInteractDistance
local CreateFrame = CreateFrame
local FollowUnit = FollowUnit
local GetCVar = GetCVar
local InCombatLockdown = InCombatLockdown
local SetCVar = SetCVar
local UIErrorsFrame = UIErrorsFrame
-- libs
-- local name space
local CommandList = {}
local HelpList = {}
local Events = {}
local Scripts = {}
local Private = {}
-- locals
local _isSfxOff = false
local _isSpeechOff = false
local _dumpTable = {}
local _following = nil
local _name = nil
local _eventFrame = nil
local _isFollowing = false
local _trying = false
local _nextTry = 0


SLASH_QUIETMACROS1 = "/q"
SLASH_QUIETMACROS2 = "/Q"
SlashCmdList["QUIETMACROS"] = function(commandMessage)
    if commandMessage then
        if CommandList[commandMessage] then
            CommandList[commandMessage]()
        end
    else
        CommandList.default()
    end
end


HelpList[#HelpList+1] = "    /q help -- show list of QuietMacros commands"
function CommandList.help()
    DEFAULT_CHAT_FRAME:AddMessage("Quiet Macros commands")
    for k,v in  ipairs(HelpList) do
        DEFAULT_CHAT_FRAME:AddMessage(v)
    end
end


HelpList[#HelpList+1] = "    /q c -- clear error text"
function CommandList.c()
    UIErrorsFrame:Clear()
end


HelpList[#HelpList+1] = "    /q 0 -- turn off error speech"
CommandList["0"] = function()
    SetCVar("Sound_EnableErrorSpeech", 0)
    _isSpeechOff = true
    CommandList.default = CommandList["1"]
end


HelpList[#HelpList+1] = "    /q 1 -- turn on error speech, sfx, and clear error text"
CommandList["1"] = function()
    if _isSfxOff then
        SetCVar("Sound_EnableSFX", 1)
        _isSfxOff = false
    end
    if _isSpeechOff then
        SetCVar("Sound_EnableErrorSpeech", 1)
        _isSpeechOff = false
        CommandList.default = CommandList["0"]
    end
    UIErrorsFrame:Clear()
end


HelpList[#HelpList+1] = "    /q seo -- turn sound effects off"
function CommandList.seo()
    SetCVar("Sound_EnableSFX", 0)
    _isSfxOff = true
    CommandList.default = CommandList["1"]
end


HelpList[#HelpList+1] = "    /q nsic -- no shifting in combat"
function CommandList.nsic()
    if InCombatLockdown() then
        SetCVar("autoUnshift", 0)
    end
end


HelpList[#HelpList+1] = "    /q ns -- no shifting at all"
function CommandList.ns()
    SetCVar("autoUnshift", 0)
end


HelpList[#HelpList+1] = "    /q as -- allow shifting"
function CommandList.as()
    SetCVar("autoUnshift", 1)
end


HelpList[#HelpList+1] = "    /q dump -- sends current states to chat window"
function CommandList.dump()
    DEFAULT_CHAT_FRAME:AddMessage("Sound_EnableErrorSpeech: "..GetCVar("Sound_EnableErrorSpeech"))
    DEFAULT_CHAT_FRAME:AddMessage("Sound_EnableSFX: "..GetCVar("Sound_EnableSFX"))
    DEFAULT_CHAT_FRAME:AddMessage("autoUnshift: "..GetCVar("autoUnshift"))
    DEFAULT_CHAT_FRAME:AddMessage("Following: "..tostring(_following))
end


CommandList.default = CommandList["0"]


function CommandList.follow()
    if _isFollowing then
        Private.StopFollowing()
    else
        local name, realm = UnitFullName("target")
        if name then
            _name = name
            _following = realm ~= nil and name.."-"..realm or name
            _eventFrame = _eventFrame or Private.CreateEventFrame()
            _isFollowing = true
            _trying = true
            Private.TryToFollow()
        end
    end
end


function Events.AUTOFOLLOW_BEGIN(name)
    if _isFollowing then
        if _trying then
            DEFAULT_CHAT_FRAME:AddMessage("Following "..name)
            _trying = false
        else
            Private.StopFollowing()
        end
    end
end


function Events.AUTOFOLLOW_END(...)
    if _isFollowing then
        if CheckInteractDistance(_following, 4) then
            Private.StopFollowing()
        elseif not _trying then
            AutoFollowStatus:Hide()
            _trying = true
            Private.TryToFollow()
        end
    end
end


function Scripts.OnEvent(self, event, ...)
    if Events[event] then
        Events[event](...)
    end
end


function Scripts.OnUpdate(self, elpased)
    Private.TryToFollow()
end


function Private.TryToFollow()
    if GetTime() < _nextTry then
        return
    end
    _nextTry = GetTime() + 0.25
    if _trying then
        _eventFrame:SetScript("OnUpdate", nil)
        if CheckInteractDistance(_following, 4) then
            FollowUnit(_following, true)
        else
            _eventFrame:SetScript("OnUpdate", Scripts.OnUpdate)
        end
    end
end


function Private.StopFollowing()
    _eventFrame:SetScript("OnUpdate", nil)
    DEFAULT_CHAT_FRAME:AddMessage("Deactivated following")
    _name = nil
    _following = nil
    _isFollowing = false
    _trying = false
    AutoFollowStatus:Hide()
end


function Private.CreateEventFrame()
    AutoFollowStatus:HookScript("OnEvent", Scripts.OnEvent)
    local frame = CreateFrame("Frame", "QuietMacrosEvents")
    frame:SetScript("OnEvent", Scripts.OnEvent)
    return frame
end
